package org.netkernel.mod.architecture;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;

public class MenuAccessor extends StandardAccessorImpl
{
	public MenuAccessor()
	{	this.declareThreadSafe();
	}
	
	/* renders explorer navigation bar
	 * arg:hash - (string on identifier) module list hash of the entity that this page represents
	 * arg:tool - (string pbv) a single string tool name added to end of navigation
	 * arg:breadcrumbs (IHDSNode pbv) 
	 * 	/breadcrumbs/crumb/
	 * 		@name - text display in link
	 * 		@view - link to a specific explorer view with optional entity 
	 * 		@type - present menu options for a specific entity
	 * 		@entity - entity hash behind this link
	 */
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		HDSBuilder b = new HDSBuilder();
		b.pushNode("div");
		
		String hash=null;
		if (aContext.getThisRequest().argumentExists("hash"))
		{	hash=aContext.getThisRequest().getArgumentValue("hash");
		}
		String tool=null;
		if (aContext.getThisRequest().argumentExists("tool"))
		{	tool=aContext.source("arg:tool",String.class);
		}
		IHDSNode breadcrumbs=null;
		if (aContext.getThisRequest().argumentExists("breadcrumbs"))
		{	breadcrumbs=aContext.source("arg:breadcrumbs",IHDSNode.class);
		}
		
		addExplorerTemplate(b,aContext,hash,tool,breadcrumbs);
		
		aContext.createResponseFrom(b.getRoot());
	}
	
	public static void addExplorerTemplate(HDSBuilder b, INKFRequestContext aContext, String aHash, String aTool, IHDSNode aBreadcrumbs) throws Exception
	{
		addCSS(b,"/nkse/style/css/jquery-ui-latest.css?v=6.1.1");
		addCSS(b,"/nkse/style/css/jquery-datatables-latest.css?v=6.1.1");
		addCSS(b,"/tools/ae/pub/explore.css?v=6.1.1");
		addCSS(b,"/nkse/style/css/book.css?v=6.1.1");
		
		addScript(b,"/nkse/style/js/jquery-contextmenu-latest.js?v=6.1.2");
		addScript(b,"/nkse/style/js/jquery-datatables-latest.js?v=6.1.1");
		addScript(b,"/tools/ae/pub/main.js?v=6.1.1");
		
		Utils.generateContextMenu(b, aContext, "gMenuModule", "module");
		Utils.generateContextMenu(b, aContext, "gMenuSpace", "space");
		Utils.generateContextMenu(b, aContext, "gMenuPhysical", "physical");
		Utils.generateContextMenu(b, aContext, "gMenuEp", "endpoint");
		Utils.generateContextMenu(b, aContext, "gMenuProto", "prototype");
		Utils.generateContextMenu(b, aContext, "gMenuRep", "representation");
		Utils.generateContextMenu(b, aContext, "gMenuTrace", "trace");
		Utils.generateContextMenu(b, aContext, "gMenuRequest", "request");
		
		IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
		
		String extraClass="";
		if (aContext.getThisRequest().argumentExists("extraClass"))
		{	extraClass=aContext.getThisRequest().getArgumentValue("extraClass");
		}
		
		b.pushNode("div");
		b.addNode("@class","explore-navbar-container cpWrapperHeader "+extraClass);
		b.pushNode("div");
		b.addNode("@class","container");
		b.pushNode("div");
		b.addNode("@class","row");
		b.pushNode("div");
		b.addNode("@class","col-md-12 explore-navbar");
		b.pushNode("span");
		b.pushNode("a","Explore");
		b.addNode("@href","/tools/ae/view/allModules");
		b.popNode();
		b.popNode();
		
		if (aHash!=null)
		{	String hashParts[]=org.netkernel.layer0.util.Utils.splitString(aHash, "/");
		
			String modHash=hashParts[0];
			IMetaRepresentation modMeta=modules.metadataForHash(modHash);
			addMeta(b,modHash,modMeta);
			
			if (hashParts.length>1)
			{	String spaceHash=modHash+"/"+hashParts[1];
				IMetaRepresentation spaceMeta=modules.metadataForHash(spaceHash);
				addMeta(b,spaceHash,spaceMeta);
				
				if (hashParts.length>2)
				{
					String elHash=spaceHash+"/"+hashParts[2];
					IMetaRepresentation elMeta=modules.metadataForHash(elHash);
					
					ISpaceElements elements=((ISpaceMeta)spaceMeta).getElements();
					for (int i=0; i<elements.size(); i++)
					{	IIdentifier id=elements.getIdentifier(i);
						if (id!=null)
						{	String eId=id.toString();
							if (eId.equals(hashParts[2]))
							{	IEndpoint physicalEp=elements.getPhysicalEndpoint(i);
								String epHash=modules.hashForComponent(physicalEp);
								IMetaRepresentation epMeta=modules.metadataForHash(epHash);
								addMeta(b,epHash,epMeta);
								break;
							}
						}
					}
					addMeta(b,elHash,elMeta);
					
				}
			}
		}
		if (aTool!=null)
		{
			addSeparator(b);
			b.addNode("span",aTool);
		}
		if (aBreadcrumbs!=null)
		{
			for (IHDSNode crumb: aBreadcrumbs.getNodes("/breadcrumbs/crumb"))
			{
				//b.pushNode("td"); b.pushNode("span", " / "); b.addNode("@class", "separator"); b.popNode(); b.popNode();
				addSeparator(b);
				String name=(String)crumb.getFirstValue("@name");
				String view=(String)crumb.getFirstValue("@view");
				String type=(String)crumb.getFirstValue("@type");
				String entity=(String)crumb.getFirstValue("@entity");
				if (view!=null)
				{	b.pushNode("td");
					b.pushNode("a",name);
					if (entity==null)
					{	b.addNode("@onclick", "doMenu('"+view+"')");
					}
					else
					{	b.addNode("@onclick", "doMenu('"+view+"','"+entity+"')");
					}
					b.addNode("@href", "#");
					b.popNode();
					b.popNode();
				}
				else if (type!=null)
				{	addEntity(b,Utils.menuClassForType(type),Utils.iconClassForType(type),entity,name);
				}
				else
				{	b.addNode("td",name);
				}
				
			}
		}
		
		b.popNode();
		b.popNode();
		b.popNode();
		b.popNode();
	}
	
	private static void addSeparator(HDSBuilder b)
	{	b.pushNode("span"); b.addNode("@class", "fa-stack");
			b.pushNode("span"," "); b.addNode("@class", "fa fa-square fa-stack-1x"); b.addNode("@style", "color: #FFFFFF; font-size: 130%;"); b.popNode();
			b.pushNode("span"," "); b.addNode("@class", "fa fa-chevron-right fa-stack-1x"); b.addNode("@style", "color: #404040;"); b.popNode();
		b.popNode();
		
	}

	private static void addEntity(HDSBuilder b, String aCSSClass, String aIcon, String aEntity, String aName)
	{
		b.pushNode("span");
		b.pushNode("a");
		b.addNode("@class", "el "+aCSSClass);
		b.addNode("@entity", aEntity);
		b.pushNode("span"," "); b.addNode("@class", aIcon); b.popNode();
		b.addNode("span", aName);
		b.popNode();
		b.popNode();
	}
	
	private static void addMeta(HDSBuilder b, String hash, IMetaRepresentation meta)
	{
		addSeparator(b);
		
		b.pushNode("td");
		b.pushNode("a");
		b.addNode("@class", "el "+Utils.menuClassForEntity(meta));
		b.addNode("@entity", hash);
		b.pushNode("span"," "); b.addNode("@class", Utils.iconClassForEntity(meta)); b.popNode();
		b.addNode("span", meta.getName());
		b.popNode();
		b.popNode();
	}
	
	private static void addScript(HDSBuilder b, String aURL)
	{	b.pushNode("script","_");
		b.addNode("@type","text/javascript");
		b.addNode("@src",aURL);
		b.popNode();
	}
	
	private static void addCSS(HDSBuilder b, String aURL)
	{	b.pushNode("link");
		b.addNode("@type","text/css");
		b.addNode("@rel","stylesheet");
		b.addNode("@href",aURL);
		b.popNode();
	}	
}
