package org.netkernel.mod.architecture.bits;

import java.util.*;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.mod.architecture.Utils;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.ISpaceWithIdentity;
import org.w3c.dom.Document;

public class ModulePhysicalStructure extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String entityHash=aContext.getThisRequest().getArgumentValue("entityId");
		
		//get public root spaces
		IRepDeployedModules modules=aContext.source("active:moduleList",IRepDeployedModules.class);
		IModule module=(IModule)modules.componentForHash(entityHash);
		List<String> spaceHashes=new ArrayList<String>();
		for (ISpaceWithIdentity space: module.getPublicSpaces())
		{ 	String spaceHash=modules.hashForComponent(space);
			spaceHashes.add(spaceHash);
			//url=url+"&root="+spaceHash;
		}
		//find any spaces with transports
		for (ISpace space: module.getSpaces())
		{	String spaceHash=modules.hashForComponent(space);
			if (spaceHashes.contains(spaceHash)) continue;
			ISpaceMeta spaceMeta=(ISpaceMeta)modules.metadataForHash(spaceHash);
			ISpaceElements el=spaceMeta.getElements();
			IEndpoint lastEp=null;
			for (int i=0; i<el.size(); i++)
			{	IEndpoint ep=el.getPhysicalEndpoint(i);
				if (ep!=lastEp)
				{	lastEp=ep;
					String epHash=modules.hashForComponent(ep);
					IMetaRepresentation epMeta=modules.metadataForHash(epHash);
					if (epMeta.getAdditionalFields().getValue(StandardTransportImpl.STATE_TRANSPORT_INDICATOR)!=null)
					{	spaceHashes.add(spaceHash);
						break;
					}
				}
			}
		}
		
		HDSBuilder b=new HDSBuilder();
		b.addNode("title",module.getMeta().getName());
		b.addNode("depth","module");
		for (String hash: spaceHashes)
		{	b.addNode("root",hash);
		}
		
		INKFRequest req=aContext.createRequest("active:StaticStructureDiagramView");
		req.addArgumentByValue("defaults", b.getRoot());
		INKFResponseReadOnly respIn=aContext.issueRequestForResponse(req);
		aContext.createResponseFrom(respIn);
	}
}
