package org.netkernel.mod.ide;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.meta.IModuleMeta;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.mod.hds.IHDSMutator;
import org.netkernel.mod.hds.IHDSReader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.ISpaceWithIdentity;
import org.netkernel.util.Utils;

public class ViewPhysicalModuleResources extends StandardAccessorImpl
{
	private final Set<String> mExcluded=new HashSet<>();
	
	public ViewPhysicalModuleResources()
	{
		mExcluded.add("class");
		mExcluded.add("png");
		mExcluded.add("jpg");
		mExcluded.add("gif");
		mExcluded.add("jar");
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String loadResource=aContext.source("httpRequest:/param/loadResource",String.class);
		if (loadResource!=null && loadResource.length()>0)
		{	onLoadResource(loadResource,aContext);
		}
		else
		{	
			String newResource=aContext.source("httpRequest:/param/newResource",String.class);
			if (newResource!=null && newResource.length()>0)
			{	onNewResource(newResource,aContext);
			}
			else
			{
				String deleteResource=aContext.source("httpRequest:/param/deleteResource",String.class);
				if (deleteResource!=null && deleteResource.length()>0)
				{	onDeleteResource(deleteResource,aContext);
				}
				else
				{
				
					String saveResource=aContext.source("httpRequest:/param/saveResource",String.class);
					if (saveResource!=null && saveResource.length()>0)
					{	
						String fileContents=aContext.source("httpRequest:/param/contents",String.class);
						onSaveResource(saveResource,fileContents,aContext);
					}
					try
					{	onListPage(aContext);
					}
					catch (Exception e)
					{
						aContext.logRaw(INKFLocale.LEVEL_WARNING, Utils.throwableToString(e));
						String msg="Problem viewing this module - is it an encrypted module? See log for details.";
						String rep="<div><div class='alert alert-danger'>"+msg+"</div></div>";
						INKFResponse resp=aContext.createResponseFrom(rep);
						resp.setHeader("WrappedControlPanel", "Browse module files");
						resp.setHeader("WrappedControlPanelSubtitle", msg);
						resp.setHeader("WrappedControlPanelIcon", "/org/netkernel/mod/ide/pub/icon.png");	
					}
				}
			}
		}
	}
	
	private String getModuleHash(INKFRequestContext aContext) throws Exception
	{
		String hash=aContext.getThisRequest().getArgumentValue("entityId");
		int i=hash.indexOf('/');
		String moduleHash=i<0?hash:hash.substring(0, i);
		return moduleHash;
	}
	
	private String getPath(INKFRequestContext aContext) throws Exception
	{
		String hash=aContext.getThisRequest().getArgumentValue("entityId");
		int i=hash.indexOf('/');
		String path=i<0?"module.xml":hash.substring(i+1);
		return path;
	}
	
	public void onDeleteResource(String aFile, INKFRequestContext aContext) throws Exception
	{
		String hash=getModuleHash(aContext);
		IRepDeployedModules modules=(IRepDeployedModules)aContext.source("active:moduleList",IRepDeployedModules.class);
		IModuleMeta meta=(IModuleMeta)modules.metadataForHash(hash);
		String sourceURI=BootUtils.fixURIString(meta.getSource().toString())+aFile;
		String error=null;
		try
		{	if (!aContext.delete(sourceURI))
			{	error="File cannot be deleted - please check permissions";
			}
		}
		catch (NKFException e)
		{	error=e.getDeepestId()+": "+e.getDeepestMessage();
		}
		if (error==null)
		{	aContext.createResponseFrom("{}").setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
		else
		{	aContext.createResponseFrom("{\"error\": \""+error+"\"}").setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
	}
	
	
	
	public void onNewResource(String aFile, INKFRequestContext aContext) throws Exception
	{
		//what can go wrong
		// -file exists
		// -filename bad and can't be created
		String hash=getModuleHash(aContext);
		IRepDeployedModules modules=(IRepDeployedModules)aContext.source("active:moduleList",IRepDeployedModules.class);
		IModuleMeta meta=(IModuleMeta)modules.metadataForHash(hash);
		String sourceURI=BootUtils.fixURIString(meta.getSource().toString())+aFile;
		String error=null;
		if (aContext.exists(sourceURI))
		{	error="File with that name exists";
		}
		else
		{	try
			{	aContext.sink(sourceURI, "");
			}
			catch (NKFException e)
			{	error=e.getDeepestId()+": "+e.getDeepestMessage();
			}
		}
		if (error==null)
		{	aContext.createResponseFrom("{}").setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
		else
		{	aContext.createResponseFrom("{\"error\": \""+error+"\"}").setExpiry(INKFResponse.EXPIRY_ALWAYS);
		}
	}
	
	public void onSaveResource(String aFile, String aContents, INKFRequestContext aContext) throws Exception
	{	
		String hash=getModuleHash(aContext);
		IRepDeployedModules modules=(IRepDeployedModules)aContext.source("active:moduleList",IRepDeployedModules.class);
		IModuleMeta meta=(IModuleMeta)modules.metadataForHash(hash);
		String sourceURI=BootUtils.fixURIString(meta.getSource().toString())+aFile;
		aContext.sink(sourceURI, aContents);
		
		aContext.createResponseFrom("done").setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

	public void onLoadResource(String aBody, INKFRequestContext aContext) throws Exception
	{
		String path=aBody;		
		String hash=getModuleHash(aContext);
		
		IRepDeployedModules modules=(IRepDeployedModules)aContext.source("active:moduleList",IRepDeployedModules.class);
		IModuleMeta meta=(IModuleMeta)modules.metadataForHash(hash);
		IModule module=(IModule)modules.componentForHash(hash);
		
		String sourceURI=BootUtils.fixURIString(meta.getSource().toString())+path;
		INKFResponseReadOnly respIn=aContext.sourceForResponse(sourceURI,IBinaryStreamRepresentation.class);
		INKFResponse resp=aContext.createResponseFrom(respIn);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
				
	}

	public void onListPage(INKFRequestContext aContext) throws Exception
	{
		String hash=getModuleHash(aContext);
		String path=getPath(aContext);
		//System.out.println(path);
		
		IRepDeployedModules modules=(IRepDeployedModules)aContext.source("active:moduleList",IRepDeployedModules.class);
		IModuleMeta meta=(IModuleMeta)modules.metadataForHash(hash);
		IModule module=(IModule)modules.componentForHash(hash);
		ISpaceWithIdentity[] spaces=module.getPublicSpaces();
		if (spaces.length>0)
		{
			INKFRequest req=aContext.createRequest("active:sls");
			req.addArgumentAsAbsolute("space", spaces[0].getIdentifier().toString());
			req.addArgumentAsAbsolute("version", module.getMeta().getVersion());
			req.setRepresentationClass(IHDSDocument.class);
			IHDSReader r=((IHDSDocument)aContext.issueRequest(req)).getReader();
			
			//System.out.println(r);
			IHDSMutator m=org.netkernel.mod.hds.HDSFactory.newDocument();
			if (((Double)r.getFirstValue("count(/*)"))>1.0)
			{	recurseDir(r,m,0);
			}
			else
			{	recurseDir(r.getFirstNode("/dir"),m,0);
			}
			
			//System.out.println(m);
						
			//check if we can modify files
			String sourceURI=BootUtils.fixURIString(meta.getSource().toString()+"module.xml");
			boolean writable=false;
			if (sourceURI.startsWith("file:"))
			{	File f=new File(URI.create(sourceURI));
				writable=f.canWrite();
			}
			m.setCursor("/dir").addNode("@writable", Boolean.toString(writable));
			m.addNode("@initialFile", path);

			req=aContext.createRequest("active:xslt");
			req.addArgument("operator","res:/org/netkernel/mod/ide/styleResourceTree.xsl");
			req.addArgumentByValue("operand", m.toDocument(false));
			req.addArgumentByValue("hash", hash);
			INKFResponseReadOnly respIn=aContext.issueRequestForResponse(req);
			
			INKFRequest xrlReq=aContext.createRequest("active:xrl2");
			xrlReq.addArgumentFromResponse("template", respIn);
			respIn=aContext.issueRequestForResponse(xrlReq);
			
			INKFResponse resp=aContext.createResponseFrom(respIn);
			
			HDSBuilder b=new HDSBuilder();
			b.pushNode("view");
			b.addNode("title","Browse module files");
			b.addNode("subtitle", "for "+meta.getName());
			b.pushNode("icon","fa fa-pencil-square-o"); b.addNode("@type", "fa"); b.popNode();
			
			resp.setHeader("WrappedControlPanel", true);
			resp.setHeader("WrappedControlHeading", b.getRoot());
			
		}
	}
	
	private int recurseDir(IHDSReader aR, IHDSMutator aM, int aOrder)
	{
		String name=(String)aR.getFirstValueOrNull("@name");
		if (name==null) name="";
		
		int order=aOrder;
		aM.pushNode("dir");
		aM.addNode("@order", Integer.toString(order++));
		aM.addNode("@name",name);
		for (IHDSReader dir : aR.getNodes("dir"))
		{	order=recurseDir(dir,aM,order);
		}
		for (IHDSReader res : aR.getNodes("res"))
		{	aM.pushNode("res");
			name=(String)res.getFirstValueOrNull("@name");
			aM.addNode("@name",name);
			aM.addNode("@id",res.getFirstValue("@id"));
			aM.addNode("@order", Integer.toString(order++));
			
			String suffix;
			int i=name.lastIndexOf('.');
			if (i>=0)
			{	suffix=name.substring(i+1);
			}
			else
			{	suffix="";
			}
			boolean excluded=mExcluded.contains(suffix);
			aM.addNode("excluded", Boolean.toString(excluded));
			aM.popNode();
		}
		
		aM.popNode();
		return order;
		
		
	}
	
}
