package tools.config;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.boot.BootPersistence;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.io.StringReader;
import java.util.*;

public class EditConfigAccessor extends StandardAccessorImpl
{
	private List<String> mNewProperties;
	
	public EditConfigAccessor()
	{	//declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode config=aContext.source("netkernel:/config",IHDSNode.class);
		HDSBuilder b = new HDSBuilder();
		b.importChildren(config);

		setDefault("se.concurrentcache.costthreshold","500",b);
		setDefault("netkernel.poll.internal","500",b);
		setDefault("netkernel.init.stabilityThreshold","10",b);
		
		addThemeSettings(b,aContext);
		
		IHDSNodeList items=b.getRoot().getNodes("/config/item");
		
		IHDSNode params=(IHDSNode)aContext.source("httpRequest:/params", IHDSNode.class);
		if (params.getChildren().length>0)
		{
			//System.out.println(params);
			
			if (params.getFirstNode("/savetheme")!=null)
			{	saveThemeSettings(params,b,items, aContext);
			}
			else
			{
				saveRegularSettings(params,b,items, aContext);
			}
			
			
		}
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/config/styleConfig.xsl");
		req.addArgumentByValue("operand", b.getRoot());
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
	}
	
	private void saveRegularSettings(IHDSNode params, HDSBuilder b, IHDSNodeList items, INKFRequestContext aContext) throws Exception
	{
		IKernel k=aContext.getKernelContext().getKernel();
		String p=aContext.transrept(BootPersistence.sourceBootResource("kernel.properties", k),String.class);
		String[] ps=Utils.splitString(p, "\n");
		String error;
		boolean hasErrors=false;
		mNewProperties=new ArrayList<String>(4);
		
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.scheduler.threadcount",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.scheduler.thread.timeout",params,items,b);			
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.scheduler.maxstackdepth",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.exception.javastack",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.poll",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.poll.internal",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"stdmod.initialisation.hysteresis",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.init.stabilityThreshold",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.statistics.period",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,false,"netkernel.statistics.window",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,true,"se.concurrentcache.maxsize",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,true,"se.concurrentcache.costthreshold",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,true,"se.resolutioncache.size",params,items,b);
		hasErrors|=updatePropertyPositiveInt(ps,true,"netkernel.init.runlevel",params,items,b);
		hasErrors|=updateProperty(ps,"http.proxyHost",(String)params.getFirstValue("http.proxyHost"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyPort",(String)params.getFirstValue("http.proxyPort"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyUsername",(String)params.getFirstValue("http.proxyUsername"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyPassword",(String)params.getFirstValue("http.proxyPassword"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyNTWorkstation",(String)params.getFirstValue("http.proxyNTWorkstation"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyDomain",(String)params.getFirstValue("http.proxyDomain"),items,b,null);
		hasErrors|=updateProperty(ps,"http.proxyExclude",(String)params.getFirstValue("http.proxyExclude"),items,b,null);

		String modulesdir=(String)params.getFirstValue("netkernel.init.modulesdir");
		hasErrors|=updateProperty(ps,"netkernel.init.modulesdir",modulesdir,items,b,null);

		String instance=(String)params.getFirstValue("netkernel.instance.identifier");
		hasErrors|=updateProperty(ps,"netkernel.instance.identifier",instance,items,b,null);
		
		String debugString=(String)params.getFirstValue("netkernel.debug");
		debugString=Boolean.toString(debugString!=null);
		hasErrors|=updateProperty(ps,"netkernel.debug",debugString,items,b,null);
		
		//cache headroom
		String headroom=(String)params.getFirstValue("se.concurrentcache.headroomMb");
		long max=Utils.getOldGen().getUsage().getMax();
		int m1=(int)(max*5/(1024*1024*100));
		int m2=(int)(max*50/(1024*1024*100));
		error=null;
		try
		{	int i = Integer.parseInt(headroom);
		
			if (i>m2 || i<m1)
			{	error="se.concurrentcache.headroomMb should be between "+m1+" and "+m2+" based on current heap configuration";
			}
		}
		catch (NumberFormatException e)
		{	error="se.concurrentcache.headroomMb must be a positive integer";
		}
		hasErrors|=updateProperty(ps,"se.concurrentcache.headroomMb",headroom,items,b,error);
		
		//cull% headroom
		String cullpercent=(String)params.getFirstValue("se.concurrentcache.cull%");
		m1=10;
		m2=100;
		error=null;
		try
		{	int i = Integer.parseInt(cullpercent);
		
			if (i>m2 || i<m1)
			{	error="se.concurrentcache.cull% should be between "+m1+" and "+m2;
			}
		}
		catch (NumberFormatException e)
		{	error="se.concurrentcache.cull% must be a positive integer";
		}
		hasErrors|=updateProperty(ps,"se.concurrentcache.cull%",cullpercent,items,b,error);
		
		if (!hasErrors)
		{
			StringBuilder sb=new StringBuilder(1024);
			for (int i=0; i<ps.length; i++)
			{	sb.append(ps[i]);
				sb.append('\n');
			}
			for (int i=0; i<mNewProperties.size(); i++)
			{	sb.append(mNewProperties.get(i));
				sb.append('\n');
			}
			IBinaryStreamRepresentation bs=aContext.transrept(sb.toString(), IBinaryStreamRepresentation.class);
			BootPersistence.sinkBootResource("kernel.properties", bs, k);
		}
		
	}
	
	
	private void setDefault(String aKey, String aValue, HDSBuilder b)
	{
		IHDSNode n=b.getRoot().getNodes("/config/item").filter(HDSPredicateFactory.namedChildStringEquals("name", aKey)).getFirstNode();
		if (n==null)
		{	IHDSNode c=b.getRoot().getFirstNode("config");
			b.setCursor(c);
			b.pushNode("item");
			b.addNode("name", aKey);
			b.addNode("value", aValue);
		}		
	}

	private boolean updatePropertyPositiveInt(String[] aPropsFile, boolean aZeroOk, String aKey, IHDSNode aParams, IHDSNodeList aItems, HDSBuilder b)
	{
		String value=(String)aParams.getFirstValue(aKey);
		String error=null;
		try
		{	int valueInt = Integer.parseInt(value);
			if (valueInt<0 || (!aZeroOk && valueInt==0))
			{	error=aKey+" must be a positive integer";
			}
		}
		catch (NumberFormatException e)
		{	error=aKey+" must be a positive integer";
		}
		return updateProperty(aPropsFile,aKey,value,aItems,b,error);
	}

	
	private boolean updateProperty(String[] aPropsFile, String aKey, String aValue, IHDSNodeList aItems, HDSBuilder b, String aValidationError)
	{	boolean found=false;
		for (int i=0; i<aPropsFile.length; i++)
		{	if (aPropsFile[i].startsWith(aKey))
			{	aPropsFile[i]=aKey+"="+aValue;
				found=true;
				break;
			}
		}
		if (!found)
		{	//new property to append to file
			mNewProperties.add(aKey+"="+aValue);
		}
		
		updateFormState(aKey,aValue,aItems,b,aValidationError);
		
		return aValidationError!=null;
	}
	
	private void updateFormState(String aKey, String aValue, IHDSNodeList aItems, HDSBuilder b, String aValidationError)
	{
		IHDSNode n=aItems.filter(HDSPredicateFactory.namedChildStringEquals("name", aKey)).getFirstNode();
		if (n!=null)
		{	IHDSNode v=n.getFirstNode("value");
			b.setCursor(v);
			b.setValue(aValue);
			if (aValidationError!=null)
			{	b.setCursor(n);
				b.addNode("error", aValidationError);
			}
		}
		else
		{	IHDSNode c=b.getRoot().getFirstNode("config");
			b.setCursor(c);
			b.pushNode("item");
			b.addNode("name", aKey);
			b.addNode("value", aValue);
			if (aValidationError!=null)
			{	b.addNode("error", aValidationError);
			}
		}
	}
	
	
	private void addThemeSettings(HDSBuilder b, INKFRequestContext aContext) throws Exception
	{
		Properties prop=new Properties();
		try
		{	//load defaults
			String st=aContext.source("res:/nkse/style/css/theme.properties",String.class);
			prop.load(new StringReader(st));

			//load override
			IReadableBinaryStreamRepresentation bs=BootPersistence.sourceBootResource("theme.properties",aContext.getKernelContext().getKernel());
			st=aContext.transrept(bs,String.class);
			prop.load(new StringReader(st));
			
		} catch (Exception e) { ;}
		
		setDefault("theme.headerLeft", prop.getProperty("header-left","#303F9F"), b);
		setDefault("theme.headerRight", prop.getProperty("header-right","#B13F8C"),b );
		setDefault("theme.headerText", prop.getProperty("header-text","#FFFFFF"), b );
		setDefault("theme.highlight", prop.getProperty("highlight","#F0F0F0"), b );
	}
	
	private void saveThemeSettings(IHDSNode params, HDSBuilder b, IHDSNodeList aItems, INKFRequestContext aContext) throws Exception
	{
		String left=(String)params.getFirstValue("theme.headerLeft");
		String right=(String)params.getFirstValue("theme.headerRight");
		String text=(String)params.getFirstValue("theme.headerText");
		String highlight=(String)params.getFirstValue("theme.highlight");
		
		updateFormState("theme.headerLeft",left,aItems,b,null);
		updateFormState("theme.headerRight",right,aItems,b,null);
		updateFormState("theme.headerText",text,aItems,b,null);
		updateFormState("theme.highlight",highlight,aItems,b,null);
		
		StringBuilder sb=new StringBuilder();
		sb.append("header-left="+left+"\n");
		sb.append("header-right="+right+"\n");
		sb.append("header-text="+text+"\n");
		sb.append("highlight="+highlight+"\n");
		IBinaryStreamRepresentation bs=aContext.transrept(sb.toString(), IBinaryStreamRepresentation.class);
		IKernel k=aContext.getKernelContext().getKernel();
		BootPersistence.sinkBootResource("theme.properties", bs, k);
		k.getRepresentationCache().clear();
		
	}
	
}