<?xml version="1.0" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" />
	<xsl:param name="param" />

	<xsl:template match="/config" xmlns:xrl="http://1060.org/xrl">
		<div>
			<xsl:if test="//error">
			<div class="alert alert-danger"><h3 style="margin-top: 0px;">Errors</h3>
					<p>Settings cannot be changed until error values corrected</p></div>
			</xsl:if>
			<form method="post">
			
				<div class="panel panel-default">
					<div class="panel-heading">Kernel Settings</div>
					<div class="panel-body">
						<div class="form-horizontal">
							<xsl:call-template name="item">
								<xsl:with-param name="label">Thread Pool</xsl:with-param>
								<xsl:with-param name="data">netkernel.scheduler.threadcount</xsl:with-param>
								<xsl:with-param name="comment">Number of worker threads held to handle asynchronous requests.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Shutdown Grace</xsl:with-param>
								<xsl:with-param name="data">netkernel.scheduler.thread.timeout</xsl:with-param>
								<xsl:with-param name="comment">Grace period for requests to complete during shutdown or restart.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Max Request Depth</xsl:with-param>
								<xsl:with-param name="data">netkernel.scheduler.maxstackdepth</xsl:with-param>
								<xsl:with-param name="comment">Maximum allowed depth of subrequests before stack overflow</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Max Java Stacktrace</xsl:with-param>
								<xsl:with-param name="data">netkernel.exception.javastack</xsl:with-param>
								<xsl:with-param name="comment">Maximum captured stack depth of Java exceptions.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Filesystem Poll</xsl:with-param>
								<xsl:with-param name="data">netkernel.poll</xsl:with-param>
								<xsl:with-param name="comment">Millisecond period to poll filesystem resources for changes.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
							
							<xsl:call-template name="item">
								<xsl:with-param name="label">Internal Poll</xsl:with-param>
								<xsl:with-param name="data">netkernel.poll.internal</xsl:with-param>
								<xsl:with-param name="comment">Millisecond period to poll internal aggregate representations for changes.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
							
							<xsl:call-template name="item">
								<xsl:with-param name="label">Initialisation Hysteresis</xsl:with-param>
								<xsl:with-param name="data">stdmod.initialisation.hysteresis</xsl:with-param>
								<xsl:with-param name="comment">Millisecond period of validity for transient initialisation resources. (Larger value may be needed to slow systems)</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
							
							<xsl:call-template name="item">
								<xsl:with-param name="label">Initialisation Stability Threshold</xsl:with-param>
								<xsl:with-param name="data">netkernel.init.stabilityThreshold</xsl:with-param>
								<xsl:with-param name="comment">Number of invalidations of spaces during startup before warnings issues. Warnings may indicate circularity or badly configured infrastructural components.</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
							
							<xsl:call-template name="item">
								<xsl:with-param name="label">Runlevel</xsl:with-param>
								<xsl:with-param name="data">netkernel.init.runlevel</xsl:with-param>
								<xsl:with-param name="comment">Determines which modules are loaded</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
					        <xsl:call-template name="item">
					          <xsl:with-param name="label">Location for extra modules.xml files</xsl:with-param>
					          <xsl:with-param name="data">netkernel.init.modulesdir</xsl:with-param>
					          <xsl:with-param name="comment">Location for extra modules.xml files (requires reboot)</xsl:with-param>
					        </xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Instance Id</xsl:with-param>
								<xsl:with-param name="data">netkernel.instance.identifier</xsl:with-param>
								<xsl:with-param name="comment">Identifier string for this instance.</xsl:with-param>
							</xsl:call-template>
			
							<div class="form-group">
								<label for="netkernel.debug" class="col-sm-3 control-label">Debug</label>
					    		<div class="col-sm-4">
					    			<input type="checkbox" name="netkernel.debug" >
										<xsl:if test="item[name='netkernel.debug']/value='true'">
											<xsl:attribute name="checked">true</xsl:attribute>
										</xsl:if>
									</input>
								</div>
								<div class="col-sm-5" style="font-size: 80%; font-style: italic;">
									Enable development time debug with associated overhead.
								</div>
							</div>
			
							<div class="form-group">
								<label for="netkernel.threadsafe.endpoint.default" class="col-sm-3 control-label">Endpoint Threadsafe Default</label>
					    		<div class="col-sm-4">
					    			<input type="checkbox" name="netkernel.threadsafe.endpoint.default" >
										<xsl:if test="item[name='netkernel.threadsafe.endpoint.default']/value='true'">
											<xsl:attribute name="checked">true</xsl:attribute>
										</xsl:if>
									</input>
					    		</div>
								<div class="col-sm-5" style="font-size: 80%; font-style: italic;">
									If checked this property will mark all endpoints as threadsafe even if not explicitly marked as threadsafe.
								</div>
							</div>
						
						</div>
						<input type="submit" value="save settings" class="btn btn-primary"/>
					</div>
				</div>
				
				<div class="panel panel-default">
					<div class="panel-heading">Data Collection</div>
					<div class="panel-body">
						<div class="form-horizontal">
							<xsl:call-template name="item">
								<xsl:with-param name="label">Statistics Period</xsl:with-param>
								<xsl:with-param name="data">netkernel.statistics.period</xsl:with-param>
								<xsl:with-param name="comment">Millisecond period to collect kernel statistics</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Statistics Window</xsl:with-param>
								<xsl:with-param name="data">netkernel.statistics.window</xsl:with-param>
								<xsl:with-param name="comment">Number of statistics samples kept</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
						</div>
						<input type="submit" value="save settings" class="btn btn-primary"/>
					</div>
				</div>
				
				<div class="panel panel-default">
					<div class="panel-heading">Cache Configuration</div>
					<div class="panel-body">
						<div class="form-horizontal">
							<xsl:call-template name="item">
								<xsl:with-param name="label">Heap Headroom</xsl:with-param>
								<xsl:with-param name="data">se.concurrentcache.headroomMb</xsl:with-param>
								<xsl:with-param name="comment">Free Mb of heap reserved before representation cache is culled</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Max Size</xsl:with-param>
								<xsl:with-param name="data">se.concurrentcache.maxsize</xsl:with-param>
								<xsl:with-param name="comment">Number of items in representation cache before it is culled</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Cull percent</xsl:with-param>
								<xsl:with-param name="data">se.concurrentcache.cull%</xsl:with-param>
								<xsl:with-param name="comment">Percentage items of removed from representation cache in one cull</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Cost Threshold</xsl:with-param>
								<xsl:with-param name="data">se.concurrentcache.costthreshold</xsl:with-param>
								<xsl:with-param name="comment">cost of an item must be at least this high before eligible for caching</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
			
							<xsl:call-template name="item">
								<xsl:with-param name="label">Resolution Cache</xsl:with-param>
								<xsl:with-param name="data">se.resolutioncache.size</xsl:with-param>
								<xsl:with-param name="comment">Number of items in resolution cache before it is culled</xsl:with-param>
								<xsl:with-param name="type">number</xsl:with-param>
							</xsl:call-template>
						</div>
						<input type="submit" value="save settings" class="btn btn-primary"/>
					</div>
				</div>
				
				<div class="panel panel-default">
					<div class="panel-heading">Proxy Settings</div>
					<div class="panel-body">
						<div class="form-horizontal">
							<xsl:call-template name="item">
								<xsl:with-param name="label">HTTP Proxy Host</xsl:with-param>
								<xsl:with-param name="data">http.proxyHost</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">HTTP Proxy Port</xsl:with-param>
								<xsl:with-param name="data">http.proxyPort</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Authenticating Proxy Username </xsl:with-param>
								<xsl:with-param name="data">http.proxyUsername</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Authenticating Proxy Password </xsl:with-param>
								<xsl:with-param name="data">http.proxyPassword</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
								<xsl:with-param name="type">password</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Windows NTLM Proxy Workstation Name </xsl:with-param>
								<xsl:with-param name="data">http.proxyNTWorkstation</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Windows NTLM Proxy Domain Name </xsl:with-param>
								<xsl:with-param name="data">http.proxyDomain</xsl:with-param>
								<xsl:with-param name="comment">(reboot required)</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Proxy Exclusion list </xsl:with-param>
								<xsl:with-param name="data">http.proxyExclude</xsl:with-param>
								<xsl:with-param name="comment">Comma separated list of host names to exclude from proxying (reboot required)</xsl:with-param>
							</xsl:call-template>
						</div>
						<input type="submit" value="save settings" class="btn btn-primary"/>
					</div>
				</div>
				
				<div class="panel panel-default">
					<div class="panel-heading">Management Console Theme Settings</div>
					<div class="panel-body">
						<div class="alert alert-info">
							<p>These settings control the theme of the management console. This is useful
							for making your instances look distinct.</p>
							<p>You will need to flush your browser cache (<b>shift-refresh browser</b>, SHIFT-F5) for these changes to be detected.</p>
						</div>
						<div class="form-horizontal">
							<xsl:call-template name="item">
								<xsl:with-param name="label">Header Left</xsl:with-param>
								<xsl:with-param name="data">theme.headerLeft</xsl:with-param>
								<xsl:with-param name="comment">CSS left color of heading gradient</xsl:with-param>
								<xsl:with-param name="type">color</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Header Right</xsl:with-param>
								<xsl:with-param name="data">theme.headerRight</xsl:with-param>
								<xsl:with-param name="comment">CSS right color of heading gradient</xsl:with-param>
								<xsl:with-param name="type">color</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Header Text</xsl:with-param>
								<xsl:with-param name="data">theme.headerText</xsl:with-param>
								<xsl:with-param name="comment">CSS text color in heading</xsl:with-param>
								<xsl:with-param name="type">color</xsl:with-param>
							</xsl:call-template>
							<xsl:call-template name="item">
								<xsl:with-param name="label">Highlight</xsl:with-param>
								<xsl:with-param name="data">theme.highlight</xsl:with-param>
								<xsl:with-param name="comment">CSS highlight color in panels and tables</xsl:with-param>
								<xsl:with-param name="type">color</xsl:with-param>
							</xsl:call-template>
						</div>
						<input type="submit" name="savetheme" value="save settings" class="btn btn-primary"/>
					</div>
				</div>
				
				
				
			</form>
		</div>
	</xsl:template>
	
	<xsl:template name="item">
		<xsl:param name="label"/>
		<xsl:param name="data"/>
		<xsl:param name="comment"/>
		<xsl:param name="type"/>
		
		<div class="form-group">
			<label for="{data}" class="col-sm-3 control-label"><xsl:value-of select="$label"/></label>
    		<div class="col-sm-4">
				<input type="text" name="{$data}" value="{item[name=$data]/value}">
					<xsl:choose>
						<xsl:when test="$type">
							<xsl:attribute name="type"><xsl:value-of select="$type"/></xsl:attribute>
						</xsl:when>
						<xsl:when test="contains($data, 'assword')">
							<xsl:attribute name="type">password</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="type">text</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
				</input>
    		</div>
    		<div class="col-sm-5">
    			<xsl:choose>
					<xsl:when test="item[name=$data]/error">
						<div style="font-size: 100%; font-style: italic; color: red;"><xsl:value-of select="item[name=$data]/error"/></div>
					</xsl:when>
					<xsl:otherwise>
						<div style="font-size: 80%; font-style: italic;"><xsl:value-of select="$comment"/></div>
					</xsl:otherwise>
				</xsl:choose>
    		</div>
		</div>
		
	</xsl:template>

</xsl:stylesheet> 
