<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan" xmlns:nk="http://1060.org">
    <xsl:output method="xml"/>
    <xsl:variable name="imgbase">/tools/introspect/pub/</xsl:variable>
    
    <xsl:param name="httpParam"/>
    <xsl:param name="records"/>
	
	<xsl:template match="/">
		<div>
			<script language="javascript"><xsl:comment>
				function resetOffset()
				{	document.form.offset.value="0";
					document.form.submit();
				}
			</xsl:comment></script>
			<style type="text/css"><![CDATA[
				.lp { padding: 0 0.25em 0 0.25em; }
				#log-table td { padding: 0 8px; }
				.form-group { margin: 1px 4px; }
				#form { margin: 4px 0; }
				#form label { padding: 0 4px; }
			]]></style>
			
			
			
			<form name="form" class="form-inline" role="form" method="get" id="form">
			
				<div class="form-group">
				
				
					<button type="submit" class="btn btn-default" name="end" alt="start page">
						<xsl:if test="$httpParam/hds/offset=0">
							<xsl:attribute name="disabled"/>
						</xsl:if>
						<span class="fa fa-step-backward"> </span>
					</button>
					<button type="submit" class="btn btn-default" name="next" alt="previous page">
						<xsl:if test="$httpParam/hds/offset=0">
							<xsl:attribute name="disabled"/>
						</xsl:if>
						<span class="fa fa-backward"> </span>
					</button>
					<button type="submit" class="btn btn-default" name="prev" alt="next page">
						<xsl:if test="$records/log/atStart">
							<xsl:attribute name="disabled"/>
						</xsl:if>
						<span class="fa fa-forward"> </span>
					</button>
					
			
				</div>
			
				<div class="form-group">
					<label class="sr-only" for="log">Log</label>
    				<select class="form-control" name="log" id="log" onChange="resetOffset()">
    					<xsl:for-each select="/logs/log[string-length(file)>0]">
							<option value="{id}">
								<xsl:if test="$httpParam/hds/log=id">
									<xsl:attribute name="selected"/>
								</xsl:if>
								<xsl:value-of select="name"/>
							</option>
						</xsl:for-each>
    				</select>
  				</div>
  				
  				<div class="form-group">
					<label for="entries">Show</label>
    				<input class="form-control" id="records" name="records" size="3">
    					<xsl:attribute name="value">
							<xsl:choose>
								<xsl:when test="$httpParam/hds/records">
									<xsl:value-of select="$httpParam/hds/records"/>
								</xsl:when>
								<xsl:otherwise>50</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
    				</input>
  				</div>
				<div class="form-group">
					<button type="submit" class="btn btn-default">
					<span class="fa fa-refresh"> </span>refresh</button>
				</div>
					
				<input type="hidden" name="offset">
					<xsl:attribute name="value">
						<xsl:choose>
							<xsl:when test="$httpParam/hds/offset">
								<xsl:value-of select="$httpParam/hds/offset"/>
							</xsl:when>
							<xsl:otherwise>0</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				</input>
			</form>
			
			
			
			<xsl:if test="$records">
				<table id="log-table" class="table table-striped">
					<tr>
						<th/>
						<th>Time</th>
						<th>Originator</th>
						<th>Message</th>
					</tr>
						<xsl:for-each select="$records/log/record">
							<xsl:sort select="position()" data-type="number" order="descending"/>
							<xsl:call-template name="record"/>
						</xsl:for-each>
				</table>			
			</xsl:if>
			
		</div>
	</xsl:template>
	
	<xsl:template name="record">
		<tr>
			<xsl:choose>
				<xsl:when test="level='SEVERE'"><xsl:attribute name="class">error</xsl:attribute></xsl:when>
				<xsl:when test="level='WARNING'"><xsl:attribute name="class">warning</xsl:attribute></xsl:when>
			</xsl:choose>
		
			<td align="center">
			<xsl:choose>
				<xsl:when test="level='SEVERE'"><span class="fa fa-times-circle text-danger"> </span></xsl:when>
				<xsl:when test="level='WARNING'"><span class="fa fa-exclamation-triangle text-warning"> </span></xsl:when>
				<xsl:when test="level='FINEST'"><span class="fa fa-bug"> </span></xsl:when>
				<xsl:otherwise><span class="fa fa-info-circle"> </span></xsl:otherwise>
			</xsl:choose>
			</td>
			<td><div class="lp"><xsl:value-of select="translate(date,'T','&#160;')"/></div></td>
			<td><div class="lp">
				<xsl:value-of select="class"/>
			</div></td>
			<td><div class="lp" style="white-space: pre-wrap; overflow: hidden;"><xsl:value-of select="message"/></div></td>
		</tr>
	</xsl:template>
	
</xsl:stylesheet>