package tools.requesttrace2;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.INKFKernelContext;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFRequestImpl;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.nkf.impl.NKFTransportContextImpl;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.layer0.util.Utils;
import org.netkernel.mod.hds.HDSFactory;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.mod.hds.IHDSMutator;
import org.netkernel.mod.hds.IHDSReader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.scheduler.IInternalResponseMeta;
import org.netkernel.scheduler.TraceDebugCapture;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.IResolution;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.impl.NetKernelException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

public class RequestTraceAccessor extends StandardAccessorImpl
{
	private final static String sRequestTraceHistoryURI="pds:/requestTraceHistory.xml";
	private final static int HISTORY_SIZE=20;
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSReader params=aContext.source("httpRequest:/params",IHDSDocument.class).getReader();
		//System.out.println(params);
		IHDSMutator m=HDSFactory.newDocument().pushNode("config");
		
		m.pushNode("params").appendChildren(params);
		m.popNode();
		
		String action1=(String)params.getFirstValueOrNull("/action1");
		if (action1==null || action1.length()==0)
		{	
			String historySelect=(String)params.getFirstValueOrNull("/history");
			if (historySelect!=null && historySelect.length()>0)
			{	onHistorySelect(historySelect,m,aContext);
			}
			
			//change in module or page load
			onPage(m,aContext);
		}
		else if ("resolve".equals(action1))
		{	try
			{	saveHistory(params,aContext);
			} catch (Exception e) {;}
			onResolve(params,m,aContext);
			onPage(m,aContext);
		}
		else if ("response".equals(action1))
		{	try
			{	saveHistory(params,aContext);
			} catch (Exception e) {;}
			onResponse(params,m,aContext);
			onPage(m,aContext);
		}
		else if ("display".equals(action1))
		{	try
			{	saveHistory(params,aContext);
			} catch (Exception e) {;}
			onDisplay(aContext, params);
		}
		
		else if ("visualize".equals(action1))
		{	try
			{	saveHistory(params,aContext);
			} catch (Exception e) {;}
			onVisualize(aContext, params);
		}
		
		//System.out.println(m);
		
	}
	
	private void onPage(IHDSMutator aPageConfig, INKFRequestContext aContext) throws Exception
	{
		INKFRequest req = aContext.createRequest("active:xslt");
        req.addArgument("operator","res:/tools/requesttrace2/styleTrace.xsl");
        req.addArgumentByValue("param", aPageConfig.toDocument(false));
        req.addArgumentByValue("history", getHistory(aContext));
	    req.addArgument("operand","active:moduleListDoc");
        INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
	    INKFResponse respOut=aContext.createResponseFrom(resp);
	    respOut.setHeader("WrappedControlPanel",true);
	    respOut.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	    respOut.setHeader("httpResponse:/header/Cache-Control","no-cache");
	}
	
	/* history document structure
	 * 
	 * <history>
	 *   <trace>
	 *     <moduleURI/>
	 *     <moduleVersion/>
	 *     <spaceIndex/>
	 *     <identifier/>
	 *     <verb/>
	 *     (OR) <declreq/>
	 */
	
	private void onHistorySelect(String aHistoryId, IHDSMutator m, INKFRequestContext aContext ) throws Exception
	{
		//System.out.println("history "+aHistoryId);
		IHDSReader history=getHistory(aContext).getReader();
		IHDSReader selected=history.getFirstNode("/history/trace["+aHistoryId+"]");
		//System.out.println(m);
		
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
		String moduleURI=(String)selected.getFirstValue("moduleURI");
		String moduleVersion=(String)selected.getFirstValue("moduleVersion");
		String spaceIndex=(String)selected.getFirstValue("spaceIndex");
		for (IHDSNode moduleNode : mods.getHierarchy().getNodes("/modules/module"))
		{	//System.out.println(moduleNode);
			if (moduleNode.getFirstValue("id").equals(moduleURI)
					&& moduleNode.getFirstValue("version").equals(moduleVersion))
			{
				String moduleHash=(String)moduleNode.getFirstValue("hash");
				IModule module=(IModule)mods.componentForHash(moduleHash);
				String spaceHash=getSpaceHashForIndex(spaceIndex, moduleHash, mods);
				/*
				ISpace space=module.getSpaces()[Integer.parseInt(spaceIndex)];
				String spaceHash=mods.hashForComponent(space);
				*/
				//System.out.println(moduleHash+" "+spaceHash);
				m.setCursor("/config/params/module").setValue(moduleHash);
				m.setCursor("/config/params/space").setValue(spaceHash);
				
				break;
			}
		}
		
		//now get the request details
		String declreq=(String)selected.getFirstValueOrNull("declreq");
		if (declreq!=null)
		{	m.setCursor("/config/params/mode").setValue("declarative");
			m.setCursor("/config/params/declreq").setValue(declreq);
		}
		else
		{
			m.setCursor("/config/params/mode").setValue("simple");
			String identifier=(String)selected.getFirstValueOrNull("identifier");
			String verb=(String)selected.getFirstValueOrNull("verb");
			m.setCursor("/config/params/identifier").setValue(identifier);
			m.setCursor("/config/params/verb").setValue(verb);
		}
		
		
	}
	
	
	private void saveHistory(IHDSReader params, INKFRequestContext aContext) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
		String spaceHash=(String)params.getFirstValue("space");
		ISpace space=(ISpace)mods.componentForHash(spaceHash);
		
		String moduleHash=(String)params.getFirstValue("module");
		IModule module=(IModule)mods.componentForHash(moduleHash);
		
		String moduleURI=module.getMeta().getIdentifier();
		String moduleVersion=module.getMeta().getVersion();
		int spaceIndex = getSpaceIndex(spaceHash, moduleHash, mods);

		if (spaceIndex>=0)
		{	//only static spaces will be stored
			String spaceIndexString=Integer.toString(spaceIndex);
			String identifier=null;
			String verb=null;
			String declreq=null;
			if (isSimpleRequest(params))
			{	identifier=(String)params.getFirstValue("identifier");
				verb=(String)params.getFirstValue("verb");
			}
			else
			{	declreq=(String)params.getFirstValue("declreq");
			}
			
			IHDSReader history=getHistory(aContext).getReader();
			//find if it exists and move to top
			int foundIndex=findHistory(history,moduleURI,moduleVersion,spaceIndexString,identifier,verb,declreq);
			IHDSMutator m=history.toDocument().getMutableClone();
			if (foundIndex>=0)
			{	moveHistoryToTop(m,foundIndex);
			}
			//if not exists add it
			else
			{	
				IMetaRepresentation sm=mods.metadataForHash(spaceHash);
				addHistory(m,moduleURI,moduleVersion,spaceIndexString,identifier,verb,declreq,aContext,sm.getName());
			}
			//persist history
			persistHistory(m,aContext);
		}
	}
	
	private int getSpaceIndex(String aSpaceHash, String aModuleHash, IRepDeployedModules aModules)
	{
		for (IHDSNode moduleNode : aModules.getHierarchy().getNodes("/modules/module"))
		{
			if (moduleNode.getFirstValue("hash").equals(aModuleHash))
			{	int i=0;
				for (IHDSNode spaceNode : moduleNode.getNodes("//space"))
				{	
					if (spaceNode.getFirstValue("hash").equals(aSpaceHash))
					{
						return i;
					}
					i++;
				}
			}
		}
		return -1;
	}
	
	private String getSpaceHashForIndex(String aIndex, String aModuleHash, IRepDeployedModules aModules)
	{	for (IHDSNode moduleNode : aModules.getHierarchy().getNodes("/modules/module"))
		{
			if (moduleNode.getFirstValue("hash").equals(aModuleHash))
			{	int i=0;
				IHDSNodeList spaces=moduleNode.getNodes("//space");
				IHDSNode spaceNode=spaces.get(Integer.parseInt(aIndex));
				return (String)spaceNode.getFirstValue("hash");
			}
		}
		return "";
	}
	
	
	private IHDSDocument getHistory(INKFRequestContext aContext) throws Exception
	{	IHDSDocument result;
		try
		{	result=aContext.source(sRequestTraceHistoryURI,IHDSDocument.class);
			//throw new Exception("make new");
		}
		catch (Exception e)
		{	result=HDSFactory.newDocument().pushNode("history").toDocument(false);
		}
		return result;
	}
	
	private int findHistory(IHDSReader aHistory, String moduleURI, String moduleVersion, String spaceIndex, String identifier, String verb, String declreq)
	{
		int foundIndex=-1;
		int i=0;
		for (IHDSReader trace : aHistory.getNodes("/history/trace"))
		{
			if (trace.getFirstValue("moduleURI").equals(moduleURI))
			{
				if (trace.getFirstValue("moduleVersion").equals(moduleVersion))
				{
					if (trace.getFirstValue("spaceIndex").equals(spaceIndex))
					{	boolean requestMatch=false;
						if (identifier!=null)
						{	if (identifier.equals(trace.getFirstValueOrNull("identifier")))
							{	if (verb.equals(trace.getFirstValueOrNull("verb")))
								{	requestMatch=true;
								}
							}
						}
						else
						{	if (declreq.equals(trace.getFirstValueOrNull("declreq")))
							{	requestMatch=true;
							}	
						}
						if (requestMatch)
						{	foundIndex=i;
							break;
						}
					}
				}
			}
			i++;
		}
		return foundIndex;
	}
	
	private void moveHistoryToTop(IHDSMutator aHistory, int aIndex)
	{
		
		
		String xpathOfItemToRemove="/history/trace["+(aIndex+1)+"]";
		IHDSDocument toMove=aHistory.getFirstNode(xpathOfItemToRemove).toDocument(true);
		aHistory.setCursor(xpathOfItemToRemove).delete();
		aHistory.setCursor("/history/trace[1]").insertBefore(toMove.getReader());
	
	
		//System.out.println(aIndex);
		//System.out.println(aHistory);
	}
	
	private void addHistory(IHDSMutator aHistory, String moduleURI, String moduleVersion, String spaceIndex, String identifier, String verb, String declreq, INKFRequestContext aContext, String aSpaceName) throws Exception
	{		
		//stop history from growing unbounded
		double count=(Double)aHistory.getFirstValue("count(/history/trace)");
		int countInteger=(int)count;
		if (countInteger>=HISTORY_SIZE)
		{	aHistory.setCursor("/history/trace[last()]").delete();
		}

		IHDSMutator newTrace=HDSFactory.newDocument();
		newTrace.pushNode("trace")
			.addNode("moduleURI",moduleURI)
			.addNode("moduleVersion",moduleVersion)
			.addNode("spaceIndex",spaceIndex);
		
		String humanReadableRequest;
		if (identifier!=null)
		{
			newTrace.addNode("identifier",identifier).addNode("verb",verb);
			INKFRequest request=aContext.createRequest(identifier);
			request.setVerb(Integer.parseInt(verb));
			humanReadableRequest=RequestFactory.toShortString(((NKFRequestImpl)request).getKernelRequest());
			
		}
		else
		{	
			newTrace.addNode("declreq",declreq);
			Node xml=aContext.transrept(declreq, Document.class);
			RequestBuilder rb=new RequestBuilder(xml, aContext.getKernelContext().getKernel().getLogger());
			INKFRequest request=rb.buildRequest(aContext, null, null);
			humanReadableRequest=RequestFactory.toShortString(((NKFRequestImpl)request).getKernelRequest());
		}
		int asIndex=humanReadableRequest.lastIndexOf(" as ");
		if (asIndex>0)
		{
			humanReadableRequest=humanReadableRequest.substring(0, asIndex);
		}
		newTrace.addNode("display", humanReadableRequest+" in "+aSpaceName);
		
		if (countInteger>0)
		{	aHistory.setCursor("/history/trace[1]");
			aHistory.insertBefore(newTrace.getFirstNode("/*"));
		}
		else
		{	aHistory.setCursor("/history");
			aHistory.appendChildren(newTrace);
		}
		

			
	}
	
	private void persistHistory(IHDSMutator aHistory, INKFRequestContext aContext) throws Exception
	{	
		//System.out.println(aHistory);
		aContext.sink(sRequestTraceHistoryURI,aHistory.toDocument(false));
	}
	
	private ISpace getSpaceFromParams(IHDSReader params, IRepDeployedModules mods) throws Exception
	{	String spaceString=(String)params.getFirstValue("space");
		ISpace space=(ISpace)mods.componentForHash(spaceString);
	    if (space==null)
	    {	throw new Exception("Space not found");
	    }
	    return space;
	}
	
	private boolean isSimpleRequest(IHDSReader params)
	{	String mode=(String)params.getFirstValueOrNull("mode");
		return !(mode.equals("declarative"));
	}
	
	private INKFRequest createRequest(INKFRequestContext aContext, IHDSReader params, IRepDeployedModules mods) throws Exception
	{
		ISpace space=getSpaceFromParams(params, mods);
		
		boolean isSimple=isSimpleRequest(params);
		
		INKFRequest req;
		if (isSimple)
		{
			String identifier=(String)params.getFirstValue("identifier");
			req=aContext.createRequest(identifier);
			int verb=Integer.parseInt(params.getFirstValue("verb").toString());
	        req.setVerb(verb);
		}
		else
		{	String decreq=(String)params.getFirstValue("declreq");
			Document d=org.netkernel.xml.util.XMLUtils.parse(new StringReader(decreq));
			RequestBuilder rb=new RequestBuilder(d,aContext.getKernelContext().getKernel().getLogger());
			ClassLoader cl=space.getClassLoader();
			if (cl==null)
			{	cl=Thread.currentThread().getContextClassLoader();
			}
			req=rb.buildRequest(aContext, null, cl);
		}
		
        IRequestScopeLevel scope=RequestScopeLevelImpl.createRootScopeLevel(space);
        req.setRequestScope(scope);
        return req;
	}
	
	private void onResolve(IHDSReader params, IHDSMutator b, INKFRequestContext aContext) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
	    b.pushNode("resolution"); 
        
        TraceDebugCapture tdc = new TraceDebugCapture();
        IRequest parentReq=aContext.getKernelContext().getThisKernelRequest();
        IResolution resolution;
        try
        {	
            INKFRequest req=createRequest(aContext,params,mods);
            IRequest toResolve=((NKFRequestImpl)req).getKernelRequest();
    		IRequest resolveReq = RequestFactory.createResolveRequest(toResolve, null, tdc, parentReq, null);
        	resolution = (IResolution)aContext.getKernelContext().issueKernelRequest(resolveReq).getRepresentation();
 
        	IHDSNode resolve1=aContext.transrept(tdc, IHDSNode.class);
            IHDSDocument resolve=aContext.transrept(resolve1, IHDSDocument.class);
            b.appendChildren(resolve.getReader());
        
        	IEndpoint endpoint = resolution.getEndpoint();
        	String endpointPhysicalHash=mods.hashForComponent(endpoint);
        	
        	b.addNode("class", endpoint.getClass().getName());
        	b.addNode("toString", endpoint.toString());
        	b.addNode("physicalHash", endpointPhysicalHash);

        	IIdentifier lii=resolution.getEndpointIdentifier();
        	if (lii!=null)
        	{
	        	String logicalIdentifier=lii.toString();
	        	ISpace space=resolution.getScope().getSpace();
	        	String endpointLogicalHash=mods.hashForComponent(space)+"/"+logicalIdentifier;
	        	b.addNode("id",logicalIdentifier);
	        	b.addNode("logicalHash",endpointLogicalHash);
        	}
        	
        	
        	IRequestScopeLevel level = resolution.getScope();
        	List<ISpace> spaces=new ArrayList<ISpace>();
        	while (level!=null)
        	{	spaces.add(0, level.getSpace());
        		level=level.getParent();
        	}
        	
        	b.pushNode("scope");
        	for (int i=spaces.size()-1; i>=0; i--)
        	{	ISpace scopeSpace=spaces.get(i);
        		b.pushNode("level");
        		b.addNode("space",scopeSpace.toString());
        		String spacelHash=mods.hashForComponent(scopeSpace);
        		b.addNode("hash",spacelHash);
        		b.popNode();
        	}
        	b.popNode();

        }
        catch (NetKernelException e)
        {	
        	b.addNode("exceptionId", e.getDeepestId());
        	b.addNode("exceptionMsg", e.getDeepestMessage());
        }
        
	}
	
	private void onResponse(IHDSReader params, IHDSMutator b, INKFRequestContext aContext) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
        try
        {	
    		INKFKernelContext kc=aContext.getKernelContext();
    		ISpace space=getSpaceFromParams(params, mods);
    		INKFRequestContext tContext=new NKFTransportContextImpl(kc.getKernel(),space,this);
    		
    		INKFRequest req=createRequest(tContext,params,mods);
        	
        	INKFResponseReadOnly resp = tContext.issueRequestForResponse(req);
        
	        b.pushNode("response");
	        Object representation=resp.getRepresentation();
	        if (representation!=null)
	        {	b.addNode("repClass", representation.getClass().getName());
		        String repHash=mods.hashForClass(representation.getClass());
		        if (repHash!=null)
		        {	b.addNode("repHash", repHash);
		        	String repName=mods.metadataForHash(repHash).getName();
		        	b.addNode("repName", repName);
		        }
		        b.addNode("repToString", representation.toString());
	        }
	        else
	        {	b.addNode("repClass", "<null>");
	        	b.addNode("repName", "<null>");
	        	b.addNode("repToString", "null");
	        }
	        
	        IResponseMeta meta=((NKFResponseReadOnlyImpl)resp).getMeta();
			boolean isExpired=meta.getExpiry().isExpired(System.currentTimeMillis());
			b.addNode("isExpired", Boolean.toString(isExpired));
			String expiryString=ExpiryFactory.toString(meta.getExpiry());
			IHDSDocument expiryHDS=aContext.transrept(expiryString, IHDSDocument.class);
			b.pushNode("expiry");
			b.appendChildren(expiryHDS.getReader());
			b.popNode();
			
			b.pushNode("usermeta");
			IRequestResponseFields userMeta=meta.getUserMetaData();
			int userMetaSize=userMeta.size();
			for (int j=0; j<userMetaSize; j++ )
			{	String key = userMeta.getKey(j);
				b.pushNode("value",userMeta.getValue(j));
				b.addNode("key",key);
				b.popNode();
			}
			b.popNode();			
			
			if (meta instanceof IInternalResponseMeta)
			{	IInternalResponseMeta im=(IInternalResponseMeta)meta;
				b.addNode("cost", Integer.toString(im.getTotalCost()));
			}
	        
	        b.popNode();
        }
        catch (NetKernelException e)
        {	//throw e;
        	b.pushNode("response");
        	b.addNode("exceptionId", e.getDeepestId());
        	b.addNode("exceptionMsg", e.getDeepestMessage());
        }
        
	}
	
	private void onDisplay(INKFRequestContext aContext, IHDSReader params) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
		INKFKernelContext kc=aContext.getKernelContext();
		ISpace space=getSpaceFromParams(params, mods);
		INKFRequestContext tContext=new NKFTransportContextImpl(kc.getKernel(),space,this);
		
		INKFRequest req=createRequest(tContext,params,mods);
        INKFResponseReadOnly resp = tContext.issueRequestForResponse(req);
        INKFResponse respOut=aContext.createResponseFrom(resp);
        respOut.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	    respOut.setHeader("httpResponse:/header/Cache-Control","no-cache");
	}
	
	private void onVisualize(INKFRequestContext aContext, IHDSReader params) throws Exception
	{
		IRepDeployedModules mods=aContext.source("active:moduleList", IRepDeployedModules.class);
        
		INKFKernelContext kc=aContext.getKernelContext();
		ISpace space=getSpaceFromParams(params, mods);
		INKFRequestContext tContext=new NKFTransportContextImpl(kc.getKernel(),space,this);
		
		//reset visualizer
		INKFRequest req=aContext.createRequest("active:visualizerSetFilter");
		req.addArgumentByValue("data", "");
		aContext.issueRequest(req);
		aContext.source("active:visualizerStart");
		aContext.source("active:visualizerClear");
		try
		{	//issue request
			req=createRequest(tContext,params,mods);
	        tContext.issueRequestForResponse(req);
		}
		catch (Exception e)
		{ //ignore
		}
        
        //stop visualizer and redirect to traces page
        aContext.source("active:visualizerStop");
        INKFResponse resp=aContext.createResponseFrom("done");
        resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
        resp.setHeader("httpResponse:/redirect", "http:/tools/ae/view/visualizer");
	}
	
}
